
local AuctionatorVersion = "2.0.1";
local AuctionatorAuthor  = "Zirco";

local AuctionatorLoaded = false;
local AuctionatorInited = false;

-----------------------------------------

local recommendElements			= {};
local auctionsTabElements		= {};
local auctionsTabLeftElements	= {};

AUCTIONATOR_ENABLE_ALT		= 1;
AUCTIONATOR_OPEN_FIRST		= 0;
AUCTIONATOR_OPEN_BUY		= 0;
AUCTIONATOR_OPEN_ALL_BAGS	= 1;
AUCTIONATOR_DEF_DURATION	= "N";		-- none

local SELL_TAB		= 1;
local MORE_TAB		= 2;
local BUY_TAB 		= 3;

local MODE_LIST_ACTIVE	= 1;
local MODE_LIST_ALL		= 2;

-- saved variables - amounts to undercut

local auctionator_savedvars_defaults =
	{
	["_5000000"]			= 10000;	-- amount to undercut buyouts over 500 gold
	["_1000000"]			= 2500;
	["_200000"]				= 1000;
	["_50000"]				= 500;
	["_10000"]				= 200;
	["_2000"]				= 100;
	["_500"]				= 5;
	["STARTING_DISCOUNT"]	= 5;	-- PERCENT
	};


-----------------------------------------

local auctionator_orig_AuctionFrameTab_OnClick;
local auctionator_orig_ContainerFrameItemButton_OnClick;
local auctionator_orig_AuctionFrameAuctions_Update;
local auctionator_orig_AuctionsCreateAuctionButton_OnClick;
local auctionator_orig_CanShowRightUIPanel;
local auctionator_orig_ChatEdit_InsertLink;
local auctionator_orig_ChatFrame_OnEvent;

local gForceMsgAreaUpdate = true;


local SETTLED				= 1;
local AUCTION_POST_PENDING	= 2;
local STACK_MERGE_PENDING	= 3;
local STACK_SPLIT_PENDING	= 4;


local gAutoSelling = false;
local gAutoSellState;

local gAutoSell_ItemName;
local gAutoSell_ItemLink;
local gAutoSell_GoodStackSize;
local gAutoSell_FullStackSize;
local gAutoSell_StartPrice;
local gAutoSell_BuyoutPrice;
local gAutoSell_Hours;
local gAutoSell_targetBS;
local gAutoSell_targetCount;
local gAutoSell_AuctionNum;
local gAutoSell_NumAuctionsToCreate;
local gAutoSell_TotalItems;

local gOpenAllBags  			= AUCTIONATOR_OPEN_ALL_BAGS;
local gTimeZero;
local gTimeTightZero;

local gAutoBatch_bagID;
local gAutoBatch_slotID;
local gAutoSplitAndBatch;

local cslots = {};
local gEmptyBScached = nil;


local gJustPosted_ItemName = nil;		-- set to the last item posted, even after the posting so that message and icon can be displayed
local gJustPosted_BuyoutPrice;
local gJustPosted_StackSize;
local gJustPosted_ItemLink;

local auctionator_pending_message = nil;

local kFirstBag, kLastBag = 0, 4;

local Atr_Confirm_Proc_Yes = nil;

local gStartingTime			= time();
local gHentryTryAgain		= nil;
local gCondensedThisSession = {};

local ITEM_HIST_NUM_LINES = 20;

local gActiveAuctions = {};

local gSellPane;
local gMorePane;
local gActivePane;
local gShopPane;

local gCurrentPane;

local gHistoryItemList = {};

local ATR_NOTHING		= 0;
local ATR_BUYING		= 1;
local ATR_JUST_BOUGHT	= 2;
local ATR_NORMAL_SCAN	= 3;

local gActionState = ATR_NOTHING;

-----------------------------------------

local BS_GetCount, BS_InCslots, BS_GetEmptySlot, BS_PostAuction, BS_FindGoodStack, BS_MergeSmallStacks, BS_SplitLargeStack;

local roundPriceDown, ToTightTime, FromTightTime, monthDay;

-----------------------------------------


function Atr_EventHandler()

--	ZC_pinkmsg (event);

	if (event == "VARIABLES_LOADED")			then	Atr_OnLoad(); 					end;
	if (event == "ADDON_LOADED")				then	Atr_OnAddonLoaded(); 			end;
	if (event == "AUCTION_ITEM_LIST_UPDATE")	then	Atr_OnAuctionUpdate(); 			end;
	if (event == "AUCTION_OWNED_LIST_UPDATE")	then	Atr_OnAuctionOwnedUpdate(); 	end;
	if (event == "AUCTION_HOUSE_SHOW")			then	Atr_OnAuctionHouseShow(); 		end;
	if (event == "AUCTION_HOUSE_CLOSED")		then	Atr_OnAuctionHouseClosed(); 	end;
	if (event == "NEW_AUCTION_UPDATE")			then	Atr_OnNewAuctionUpdate(); 		end;
--	if (event == "BAG_UPDATE")					then	Atr_OnBagUpdate(); 				end;

end

-----------------------------------------

function Atr_OnLoad()

	gTimeZero		= time({year=2000, month=1, day=1, hour=0});
	gTimeTightZero	= time({year=2008, month=8, day=1, hour=0});

	AuctionatorLoaded = true;

	if ( IsAddOnLoaded("Blizzard_AuctionUI") ) then		-- need this for AH_QuickSearch since that mod forces Blizzard_AuctionUI to load at a startup
		Atr_Init();
	end

end


-----------------------------------------

local gPrevTime = 0;

function Atr_OnAddonLoaded()

	local addonName = arg1:lower();

	if (addonName == "blizzard_auctionui") then
		Atr_Init();
	end

	local now = time();

--		ZC_chatmsg (addonName.."   time: "..now - gStartingTime);

	gPrevTime = now;

end

-----------------------------------------

function Atr_Init()

	if (AuctionatorInited) then
		return;
	end

--	ZC_chatmsg("Auctionator Initialized");

	AuctionatorInited = true;

	if (AUCTIONATOR_SAVEDVARS == nil) then
		Atr_ResetSavedVars();
	end

	if (AUCTIONATOR_PRICING_HISTORY == nil) then
		AUCTIONATOR_PRICING_HISTORY = {};
	end

	if (AUCTIONATOR_SHOPPING_LISTS == nil) then
		AUCTIONATOR_SHOPPING_LISTS = {};
		Atr_SList.create ("Recent Searches", true);

		local slist = Atr_SList.create ("Sample Shopping List #1");
		slist:AddItem ("Greater Cosmic Essence");
		slist:AddItem ("Infinite Dust");
		slist:AddItem ("Dream Shard");
		slist:AddItem ("Abyss Crystal");
	else
		Atr_ShoppingListsInit();
	end

	gShopPane	= Atr_AddSellTab ("Buy",		BUY_TAB);
	gSellPane	= Atr_AddSellTab ("Sell",		SELL_TAB);
	gMorePane	= Atr_AddSellTab ("More...",	MORE_TAB);

	Atr_AddSellPanel ();

	Atr_SetupHookFunctions ();

	auctionsTabElements[1]  = AuctionsScrollFrame;
	auctionsTabElements[2]  = AuctionsButton1;
	auctionsTabElements[3]  = AuctionsButton2;
	auctionsTabElements[4]  = AuctionsButton3;
	auctionsTabElements[5]  = AuctionsButton4;
	auctionsTabElements[6]  = AuctionsButton5;
	auctionsTabElements[7]  = AuctionsButton6;
	auctionsTabElements[8]  = AuctionsButton7;
	auctionsTabElements[9]  = AuctionsButton8;
	auctionsTabElements[10] = AuctionsButton9;
	auctionsTabElements[11] = AuctionsQualitySort;
	auctionsTabElements[12] = AuctionsDurationSort;
	auctionsTabElements[13] = AuctionsHighBidderSort;
	auctionsTabElements[14] = AuctionsBidSort;
	auctionsTabElements[15] = AuctionsCancelAuctionButton;
	auctionsTabElements[16] = AuctionsTitle;

	auctionsTabLeftElements[1] = StartPrice;
	auctionsTabLeftElements[2] = AuctionsShortAuctionButton;
	auctionsTabLeftElements[3] = AuctionsMediumAuctionButton;
	auctionsTabLeftElements[4] = AuctionsLongAuctionButton;
	auctionsTabLeftElements[5] = BuyoutPrice;
	auctionsTabLeftElements[6] = AuctionsCreateAuctionButton;
	auctionsTabLeftElements[7] = AuctionsItemButton;
	auctionsTabLeftElements[8] = AuctionsDepositMoneyFrame;

	if (WOWEcon_AH_AuctionPriceHistory_Enable) then
		auctionsTabLeftElements[9] = WOWEcon_AH_AuctionPriceHistory_Enable;
		auctionsTabLeftElements[10] = WOWEcon_AH_AuctionPriceHistory_Enable_Label;
	end

	recommendElements[1] = getglobal ("Atr_Recommend_Text");
	recommendElements[2] = getglobal ("Atr_RecommendPerItem_Text");
	recommendElements[3] = getglobal ("Atr_RecommendPerItem_Price");
	recommendElements[4] = getglobal ("Atr_RecommendPerStack_Text");
	recommendElements[5] = getglobal ("Atr_RecommendPerStack_Price");
	recommendElements[6] = getglobal ("Atr_Recommend_Basis_Text");
	recommendElements[7] = getglobal ("Atr_RecommendItem_Tex");

	-- create the lines that appear in the item history scroll pane

	local line, n;

	for n = 1, ITEM_HIST_NUM_LINES do
		local y = -5 - ((n-1)*16);
		line = CreateFrame("BUTTON", "AuctionatorHEntry"..n, Atr_Hlist, "Atr_HEntryTemplate");
		line:SetPoint("TOPLEFT", 0, y);
	end

	------------------

	CreateFrame( "GameTooltip", "MyScanningTooltip" ); -- Tooltip name cannot be nil
	MyScanningTooltip:SetOwner( WorldFrame, "ANCHOR_NONE" );
	-- Allow tooltip SetX() methods to dynamically add new lines based on these
	MyScanningTooltip:AddFontStrings(
		MyScanningTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
		MyScanningTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" ) );

end

-----------------------------------------

function Atr_GetSellItemInfo ()

	local auctionItemName, auctionTexture, auctionCount = GetAuctionSellItemInfo();

	if (auctionItemName == nil) then
		auctionItemName = "";
		auctionCount	= 0;
	end

	local auctionItemLink = nil;

	-- only way to get sell itemlink that I can figure

	if (auctionItemName ~= "") then
		MyScanningTooltip:SetAuctionSellItem();
		local name;
		name, auctionItemLink = MyScanningTooltip:GetItem();

		if (auctionItemLink == nil) then
			return "",0,nil;
		end

	end

	return auctionItemName, auctionCount, auctionItemLink;

end


-----------------------------------------

function Atr_ResetSavedVars ()
	AUCTIONATOR_SAVEDVARS = ZC_CopyDeep (auctionator_savedvars_defaults);
end


--------------------------------------------------------------------------------
-- don't reference these directly; use the function below instead

local _AUCTIONATOR_SELL_TAB_INDEX = 0;
local _AUCTIONATOR_MORE_TAB_INDEX = 0;
local _AUCTIONATOR_BUY_TAB_INDEX = 0;

--------------------------------------------------------------------------------

function Atr_FindTabIndex (whichTab)

	if (_AUCTIONATOR_SELL_TAB_INDEX == 0) then

		local i = 4;
		while (true)  do
			local tab = getglobal('AuctionFrameTab'..i);
			if (tab == nil) then
				break;
			end

			if (tab.auctionatorTab) then
				if (tab.auctionatorTab == SELL_TAB)		then _AUCTIONATOR_SELL_TAB_INDEX = i; end;
				if (tab.auctionatorTab == MORE_TAB)		then _AUCTIONATOR_MORE_TAB_INDEX = i; end;
				if (tab.auctionatorTab == BUY_TAB)		then _AUCTIONATOR_BUY_TAB_INDEX = i; end;
			end

			i = i + 1;
		end
	end

	if (whichTab == SELL_TAB)	then return _AUCTIONATOR_SELL_TAB_INDEX ; end;
	if (whichTab == MORE_TAB)	then return _AUCTIONATOR_MORE_TAB_INDEX; end;
	if (whichTab == BUY_TAB)	then return _AUCTIONATOR_BUY_TAB_INDEX; end;

	return 0;
end


-----------------------------------------


function Atr_AuctionFrameTab_OnClick (self, index)

	if ( not index ) then
		index = self:GetID();
	end

	getglobal("Atr_Sell_Panel"):Hide();

	gActionState = ATR_NOTHING;		-- just in case

	if (index == 3) then
		Atr_ShowElems (auctionsTabElements);
		Atr_ShowElems (auctionsTabLeftElements);
	end

	auctionator_orig_AuctionFrameTab_OnClick (nil, index);

	if (not Atr_IsAuctionatorTab(index)) then
		gForceMsgAreaUpdate = true;
		Atr_HideAllDialogs();
		AuctionFrameMoneyFrame:Show();
		gAutoSelling = false;

		if (AP_Bid_MoneyFrame) then		-- for the addon 'Auction Profit'
			if (AP_ShowBid)	then	AP_ShowHide_Bid_Button(1);	end;
			if (AP_ShowBO)	then	AP_ShowHide_BO_Button(1);	end;
		end


	elseif (Atr_IsAuctionatorTab(index)) then
	
		AuctionFrameAuctions:Hide();
		AuctionFrameBrowse:Hide();
		AuctionFrameBid:Hide();
		PlaySound("igCharacterInfoTab");

		PanelTemplates_SetTab(AuctionFrame, index);

		AuctionFrameTopLeft:SetTexture	("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-TopLeft");
		AuctionFrameTop:SetTexture		("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-Top");
		AuctionFrameTopRight:SetTexture	("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-TopRight");
		AuctionFrameBotLeft:SetTexture	("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-BotLeft");
		AuctionFrameBot:SetTexture		("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-Bot");
		AuctionFrameBotRight:SetTexture	("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-BotRight");
		
		if (index == Atr_FindTabIndex(SELL_TAB))	then gCurrentPane = gSellPane; end;
		if (index == Atr_FindTabIndex(BUY_TAB))		then gCurrentPane = gShopPane; end;
		if (index == Atr_FindTabIndex(MORE_TAB))	then gCurrentPane = gMorePane; end;

		if (index == Atr_FindTabIndex(SELL_TAB))	then AuctionatorTitle:SetText ("Auctionator - Sell"); end;
		if (index == Atr_FindTabIndex(BUY_TAB))		then AuctionatorTitle:SetText ("Auctionator - Buy"); end;
		if (index == Atr_FindTabIndex(MORE_TAB))	then AuctionatorTitle:SetText ("Auctionator - More..."); end;




		Atr_ClearHlist();
		Atr_BatchButton:Hide();
		Atr_Hlist:Hide();
		Atr_Hlist_ScrollFrame:Hide();
		Atr_Search_Box:Hide();
		Atr_Search_Button:Hide();
		Atr_Buy1_Button:Hide();
		Atr_BatchCancelButton:Hide();
		Atr_AddToSListButton:Hide();
		Atr_RemFromSListButton:Hide();
		Atr_NewSListButton:Hide();
		Atr_DelSListButton:Hide();
		Atr_DropDown1:Hide();
		Atr_DropDownSL:Hide();

		Atr_History_RB:Show();
		Atr_CurrentAuctions_RB:Show();
		
		AuctionFrameMoneyFrame:Hide();
		
		if (index == Atr_FindTabIndex(SELL_TAB)) then
			AuctionFrameAuctions:Show();
			Atr_HideElems (auctionsTabElements);
			Atr_ShowElems (auctionsTabLeftElements);
			Atr_BatchButton:Show();
		else
			Atr_Hlist:Show();
			Atr_Hlist_ScrollFrame:Show();
			if (gJustPosted_ItemName) then
				gJustPosted_ItemName = nil;
				gSellPane:SetItem ("");
			end
		end


		if (index == Atr_FindTabIndex(MORE_TAB)) then
			FauxScrollFrame_SetOffset (Atr_Hlist_ScrollFrame, gCurrentPane.hlistScrollOffset);
			Atr_DisplayHlist();
			Atr_DropDown1:Show();
		end
		
		AuctionatorTabText:Show();
		
		if		(index == Atr_FindTabIndex(MORE_TAB))	then	AuctionatorTabText:SetText ("More...");
		elseif	(index == Atr_FindTabIndex(BUY_TAB))	then	AuctionatorTabText:SetText ("Shopping Lists");
		else
			AuctionatorTabText:Hide();
		end;
		
		if (index == Atr_FindTabIndex(BUY_TAB)) then
			Atr_Search_Box:Show();
			Atr_Search_Button:Show();
			AuctionFrameMoneyFrame:Show();
			Atr_BuildGlobalHistoryList(true);
			Atr_AddToSListButton:Show();
			Atr_RemFromSListButton:Show();
			Atr_NewSListButton:Show();
			Atr_DelSListButton:Show();
			Atr_DropDownSL:Show();
			Atr_Hlist:SetHeight (252);
			Atr_Hlist_ScrollFrame:SetHeight (252);
		else
			Atr_Hlist:SetHeight (335);
			Atr_Hlist_ScrollFrame:SetHeight (335);
		end

		if (index == Atr_FindTabIndex(BUY_TAB) or index == Atr_FindTabIndex(SELL_TAB)) then
			Atr_Buy1_Button:Show();
			Atr_Buy1_Button:Disable();
		end

		
		Atr_HideElems (recommendElements);

		getglobal("Atr_Sell_Panel"):Show();

		gCurrentPane.UINeedsUpdate = true;

		if (gOpenAllBags == 1) then
			OpenAllBags(true);
			gOpenAllBags = 0;
		end

	end

end

-----------------------------------------

function Atr_Test1 ()
	local itemName, count, itemLink = Atr_GetSellItemInfo();

	local now = time();
	local h = 3600;
	local d = 24 * h;
	local m = 30 * d;

	Atr_AddHistoricalPrice (itemName, 10000, 12, itemLink, now - 6*h)
	Atr_AddHistoricalPrice (itemName, 20000, 12, itemLink, now - 7*h)
	Atr_AddHistoricalPrice (itemName, 30000, 12, itemLink, now - 8*h)
	Atr_AddHistoricalPrice (itemName, 40000, 12, itemLink, now - 1*d)
	Atr_AddHistoricalPrice (itemName, 50000, 12, itemLink, now - 7*d)
	Atr_AddHistoricalPrice (itemName, 60000, 12, itemLink, now - (7*d + 3*h))
	Atr_AddHistoricalPrice (itemName, 70000, 12, itemLink, now - (7*d + 4*h))
	Atr_AddHistoricalPrice (itemName, 80000, 12, itemLink, now - (7*d + 5*h))
	Atr_AddHistoricalPrice (itemName, 90000, 12, itemLink, now - (7*d + 6*h))
	Atr_AddHistoricalPrice (itemName, 100000, 12, itemLink, now - (2*m + 6*d))
	Atr_AddHistoricalPrice (itemName, 110000, 12, itemLink, now - (2*m + 7*d))
	Atr_AddHistoricalPrice (itemName, 120000, 12, itemLink, now - (2*m + 12*d))
	Atr_AddHistoricalPrice (itemName, 130000, 12, itemLink, now - (2*m + 14*d))
	Atr_AddHistoricalPrice (itemName, 140000, 12, itemLink, now - 3*m)
	Atr_AddHistoricalPrice (itemName, 150000, 12, itemLink, now - 4*m)
	Atr_AddHistoricalPrice (itemName, 160000, 12, itemLink, now - 5*m)
	Atr_AddHistoricalPrice (itemName, 170000, 12, itemLink, now - 36*m)
	Atr_AddHistoricalPrice (itemName, 180000, 12, itemLink, now - 37*m)
	Atr_AddHistoricalPrice (itemName, 190000, 12, itemLink, now - 38*m)
end

-----------------------------------------

function Atr_Test2 ()
	local itemName, count, itemLink = Atr_GetSellItemInfo();

	local now = time();
	local h = 3600;
	local d = 24 * h;
	local m = 30 * d;

	local jj;

	for jj = 1,60,2 do
		Atr_AddHistoricalPrice (itemName, (jj+1000), 12, itemLink, now - (40+jj)*h);
	end
end


-----------------------------------------

function Atr_Test3 ()
	local itemName, count, itemLink = Atr_GetSellItemInfo();

	Atr_Condense_History (itemName);

end

-----------------------------------------

function Atr_IsModeCreateAuction ()
	return (Atr_IsTabSelected(SELL_TAB));
end


-----------------------------------------

function Atr_IsModeBuy ()
	return (Atr_IsTabSelected(BUY_TAB));
end


-----------------------------------------

function Atr_OnDropItem()

	if (GetCursorInfo() == "item") then
		if (Atr_IsTabSelected(SELL_TAB)) then
			ClickAuctionSellItemButton();
			ClearCursor();
			AuctionsFrameAuctions_ValidateAuction();
		end
		
		if (Atr_IsTabSelected(BUY_TAB)) then
			local iType, iID, iLink = GetCursorInfo();
			local iName = GetItemInfo (iLink);
			Atr_Search_Box:SetText (iName);
			Atr_Search_Onclick ();
			ClearCursor();
			AuctionsFrameAuctions_ValidateAuction();
			Atr_Hilite1:Hide();
		end
	end

end

-----------------------------------------

function Atr_ContainerFrameItemButton_OnModifiedClick (self, button)

	if (AUCTIONATOR_ENABLE_ALT ~= 0 and	AuctionFrame:IsShown() and IsAltKeyDown()) then
	
		local bagID  = this:GetParent():GetID();
		local slotID = this:GetID();

		if (not Atr_IsTabSelected(SELL_TAB)) then
			AuctionFrameTab_OnClick (nil, Atr_FindTabIndex(SELL_TAB));
		end

		if (IsControlKeyDown()) then
			Atr_ClearAll();
			Atr_AutoSplitAndPost (bagID, slotID);
			return;
		end

		PickupContainerItem(bagID, slotID);

		local infoType = GetCursorInfo()

		if (infoType == "item") then
			Atr_ClearAll();
			ClickAuctionSellItemButton();
			ClearCursor();
		end

		return;
	end
	
	return auctionator_orig_ContainerFrameItemButton_OnModifiedClick (self, button);

end

-----------------------------------------

function Atr_AutoSplitAndPost (bagID, slotID)

	local b1, s1 = Atr_FindOrCreateSingleton (bagID, slotID);
	
	gAutoBatch_bagID  = b1;
	gAutoBatch_slotID = s1;
	
	if (gAutoBatch_bagID == nil) then
		Atr_Error_Text:SetText ("To split off a single item you must\n\nhave at least one empty slot in your bags.");
		Atr_Error_Frame.withMask = 1;
		Atr_Error_Frame:Show ();
	end
	
end

---------------------------------------------------------

function Atr_Idle_CheckForAutoSplit ()

	if (gAutoBatch_bagID) then
		local itemLink = GetContainerItemLink(gAutoBatch_bagID, gAutoBatch_slotID);		-- check to see if singleton has "arrived"
		if (itemLink) then
			PickupContainerItem(gAutoBatch_bagID, gAutoBatch_slotID);

			ClickAuctionSellItemButton();
			ClearCursor();

			gAutoBatch_bagID  = nil;
--			gAutoSplitAndBatch = true;		-- hold off on this feature for now
		end
	end
end

-----------------------------------------

function Atr_FindOrCreateSingleton (bagID, slotID)

	local searchLink = GetContainerItemLink(bagID, slotID);
	local searchName = GetItemInfo(searchLink);
	
	local b, s, bx, sx;
	
	-- first look for a singleton

	for b = kFirstBag, kLastBag do
		local numslots = GetContainerNumSlots (b);
		for s = 1,numslots do
			local itemLink = GetContainerItemLink(b, s);
			if (itemLink == nil) then
				bx = b;
				sx = s;
			else
				local itemName = GetItemInfo(itemLink);
				if (itemName == searchName) then
					local texture, itemCount = GetContainerItemInfo(b, s);
					if (itemCount == 1) then
						return b, s;
					end
				end
			end
		end
	end

	-- split off a singleton if there's an empty slot
	
	if (bx) then
		SplitContainerItem  (bagID, slotID, 1);
		PickupContainerItem (bx, sx);
		return bx, sx;
	end
	
	-- oh well
	
	return nil, nil;
end

-----------------------------------------

function BeginAutoSell ()

	local maxStacks = math.floor (gAutoSell_TotalItems / gAutoSell_GoodStackSize);

	if (Atr_Batch_NumAuctions:GetNumber() > maxStacks) then
		Atr_Error_Text:SetText ("You can create at most "..maxStacks.." auctions");
		Atr_Error_Frame:Show ();
		return;
	end

	Atr_Batch_Frame:Hide();

--	ZC_chatmsg ("BeginAutoSell");

	gAutoSell_NumAuctionsToCreate = Atr_Batch_NumAuctions:GetNumber();

	gJustPosted_ItemName	= gAutoSell_ItemName;
	gJustPosted_BuyoutPrice	= gAutoSell_BuyoutPrice;
	gJustPosted_StackSize	= gAutoSell_GoodStackSize;
	gJustPosted_ItemLink	= gAutoSell_ItemLink;

	local bagID, slotID, numslots;

	-- build a table of all the slots that contain the item

	cslots			= {};
	gEmptyBScached	= nil;

	for bagID = kFirstBag, kLastBag do
		numslots = GetContainerNumSlots (bagID);
		for slotID = 1,numslots do
			local itemLink = GetContainerItemLink(bagID, slotID);
			if (itemLink) then
				local itemName = GetItemInfo(itemLink);
				if (itemName == gAutoSell_ItemName) then
					local bs = {};
					bs.bagID  = bagID;
					bs.slotID = slotID;
					tinsert (cslots, bs);
				end
			end
		end
	end

	-- get it going (see the idle loop)

	gAutoSellState			= SETTLED
	gAutoSelling			= true;
	gAutoSell_AuctionNum	= 1;
end




-----------------------------------------

function Atr_AuctionFrameAuctions_Update()

	auctionator_orig_AuctionFrameAuctions_Update();

	if (Atr_IsTabSelected()  and	AuctionFrame:IsShown()) then
		Atr_HideElems (auctionsTabElements);
	end


end

-----------------------------------------
-- Intercept the Create Auction click so
-- that we can note the auction values
-----------------------------------------

function Atr_AuctionsCreateAuctionButton_OnClick(...)

	if (Atr_IsTabSelected(SELL_TAB) and AuctionFrame:IsShown()) then

		if (MoneyInputFrame_GetCopper(BuyoutPrice) == 0) then
			Atr_Confirm_Text:SetText ("Are you sure you want to create\nan auction with no buyout price?");
			Atr_Confirm_Frame:Show ();
			Atr_Confirm_Proc_Yes = Atr_CreateAuction_Click;
		else
			Atr_CreateAuction_Click()
		end

	else
		auctionator_orig_AuctionsCreateAuctionButton_OnClick(...);
	end


end

-----------------------------------------

function Atr_CreateAuction_Click()

	gJustPosted_BuyoutPrice	= MoneyInputFrame_GetCopper(BuyoutPrice);
	gJustPosted_ItemName	= gCurrentPane.itemName;
	gJustPosted_StackSize	= gCurrentPane.stackSize;
	gJustPosted_ItemLink	= gCurrentPane.itemLink;

	auctionator_orig_AuctionsCreateAuctionButton_OnClick();
end

-----------------------------------------

function Atr_LogMsg (itemlink, itemcount, price)

	local logmsg = "Auction created for "..itemlink;
	if (itemcount > 1) then
		logmsg = logmsg.."x"..itemcount;
	end

	logmsg = logmsg.."   "..ZC_priceToString(price);

	ZC_chatmsg (logmsg, .9, .9, .2);

end

-----------------------------------------

function Atr_OnAuctionOwnedUpdate ()

	if (not Atr_IsTabSelected()) then
		Atr_ClearScanCache();		-- if not our tab, we have no idea what happened so must flush all caches
		return;
	end;

	gActiveAuctions = {};		-- always flush this cache

	if (gAutoSelling) then
		if (gAutoSellState == AUCTION_POST_PENDING) then
			gAutoSellState = SETTLED;
			gAutoSell_AuctionNum = gAutoSell_AuctionNum + 1;
		end
	end

	if (gAutoSelling) then

		Atr_Recommend_Text:SetText ("Auction #"..(gAutoSell_AuctionNum-1).." created for "..gAutoSell_ItemName);
		MoneyFrame_Update ("Atr_RecommendPerStack_Price", gAutoSell_BuyoutPrice);
		Atr_SetTextureButton ("Atr_RecommendItem_Tex", gAutoSell_GoodStackSize, gAutoSell_ItemLink);

		Atr_LogMsg (gAutoSell_ItemLink, gAutoSell_GoodStackSize, gAutoSell_BuyoutPrice);

		if (gAutoSell_AuctionNum-1 == gAutoSell_NumAuctionsToCreate) then
			Atr_AddHistoricalPrice (gAutoSell_ItemName, gAutoSell_BuyoutPrice / gAutoSell_GoodStackSize, gAutoSell_GoodStackSize, gAutoSell_ItemLink);

			AuctionatorAddToScan (gAutoSell_ItemName, gAutoSell_GoodStackSize, gAutoSell_BuyoutPrice, gAutoSell_NumAuctionsToCreate);

			gJustPosted_ItemName = gAutoSell_ItemName;
			gAutoSelling = false;
		else
			Atr_RedisplayAuctions();
		end

	elseif (gJustPosted_ItemName) then

		Atr_LogMsg (gJustPosted_ItemLink, gJustPosted_StackSize, gJustPosted_BuyoutPrice);

		Atr_AddHistoricalPrice (gJustPosted_ItemName, gJustPosted_BuyoutPrice / gJustPosted_StackSize, gJustPosted_StackSize, gJustPosted_ItemLink);

		AuctionatorAddToScan (gJustPosted_ItemName, gJustPosted_StackSize, gJustPosted_BuyoutPrice, 1);
	end


end

-----------------------------------------

function Atr_ResetDuration()

	if (AUCTIONATOR_DEF_DURATION == "S") then AuctionsRadioButton_OnClick(1); end;
	if (AUCTIONATOR_DEF_DURATION == "M") then AuctionsRadioButton_OnClick(2); end;
	if (AUCTIONATOR_DEF_DURATION == "L") then AuctionsRadioButton_OnClick(3); end;

end

-----------------------------------------

function AuctionatorAddToScan (itemName, stackSize, buyoutPrice, numAuctions)

	local scan = Atr_FindScan (itemName);

	scan:AddScanItem (itemName, stackSize, buyoutPrice, UnitName("player"), numAuctions);

	scan:CondenseAndSort ();

	gCurrentPane.UINeedsUpdate = true;
end

-----------------------------------------

function AuctionatorSubtractFromScan (itemName, stackSize, buyoutPrice)

	local scan = Atr_FindScan (itemName);

	scan:SubtractScanItem (itemName, stackSize, buyoutPrice);

	scan:CondenseAndSort ();

	gCurrentPane.UINeedsUpdate = true;
end

-----------------------------------------

function Atr_OnNewAuctionUpdate()


end

-----------------------------------------

function Atr_SetupHookFunctions ()

	auctionator_orig_AuctionFrameTab_OnClick = AuctionFrameTab_OnClick;
	AuctionFrameTab_OnClick = Atr_AuctionFrameTab_OnClick;

	auctionator_orig_ContainerFrameItemButton_OnModifiedClick = ContainerFrameItemButton_OnModifiedClick;
	ContainerFrameItemButton_OnModifiedClick = Atr_ContainerFrameItemButton_OnModifiedClick;

	auctionator_orig_AuctionFrameAuctions_Update = AuctionFrameAuctions_Update;
	AuctionFrameAuctions_Update = Atr_AuctionFrameAuctions_Update;

	auctionator_orig_AuctionsCreateAuctionButton_OnClick = AuctionsCreateAuctionButton_OnClick;
	AuctionsCreateAuctionButton_OnClick = Atr_AuctionsCreateAuctionButton_OnClick;

	auctionator_orig_CanShowRightUIPanel = CanShowRightUIPanel;
	CanShowRightUIPanel = auctionator_CanShowRightUIPanel;
	
	auctionator_orig_ChatEdit_InsertLink = ChatEdit_InsertLink;
	ChatEdit_InsertLink = auctionator_ChatEdit_InsertLink;
	
	auctionator_orig_ChatFrame_OnEvent = ChatFrame_OnEvent;
	ChatFrame_OnEvent = auctionator_ChatFrame_OnEvent;
end

-----------------------------------------

function auctionator_ChatEdit_InsertLink(text)

	if (AuctionFrame:IsShown() and IsShiftKeyDown() and Atr_IsTabSelected(BUY_TAB)) then	
		local item;
		if ( strfind(text, "item:", 1, true) ) then
			item = GetItemInfo(text);
		end
		if ( item ) then
			Atr_Search_Box:SetText (item);
			Atr_Search_Onclick ();
			return true;
		end
	end

	return auctionator_orig_ChatEdit_InsertLink(text);

end

-----------------------------------------

function auctionator_ChatFrame_OnEvent(self, event, ...)

	if (event == "CHAT_MSG_SYSTEM") then
		if (arg1 == ERR_AUCTION_STARTED) then		-- absorb the Auction Created message
			return;
		end
	end

	return auctionator_orig_ChatFrame_OnEvent (self, event, ...);

end

-----------------------------------------

function auctionator_CanShowRightUIPanel(frame)

	if (ZC_StringSame (frame:GetName(), "TradeSkillFrame")) then
		return 1;
	end;

	return auctionator_orig_CanShowRightUIPanel(frame);

end

-----------------------------------------

function Atr_AddSellPanel ()

	local frame = CreateFrame("FRAME", "Atr_Sell_Panel", AuctionFrame, "Atr_Sell_Template");
	frame:Hide();

	UIDropDownMenu_SetWidth (Atr_DropDownSL, 150, 5);
	UIDropDownMenu_JustifyText (Atr_DropDownSL, "CENTER");
end

-----------------------------------------

function Atr_AddSellTab (tabtext, whichTab)

	local n = AuctionFrame.numTabs+1;

	local framename = "AuctionFrameTab"..n;

	local frame = CreateFrame("Button", framename, AuctionFrame, "AuctionTabTemplate");

	frame:SetID(n);
	frame:SetText(tabtext);

	local font = frame:GetNormalFontObject();
	font:SetTextColor(1, 0.5, 0.25, 1.0);
	frame:SetNormalFontObject(font);

	frame.auctionatorTab = whichTab;

	frame:SetPoint("LEFT", getglobal("AuctionFrameTab"..n-1), "RIGHT", -8, 0);

	PanelTemplates_SetNumTabs (AuctionFrame, n);
	PanelTemplates_EnableTab  (AuctionFrame, n);
	
	return AtrPane.create (whichTab);
end

-----------------------------------------

function Atr_HideElems (tt)

	if (not tt) then
		return;
	end

	for i,x in ipairs(tt) do
		x:Hide();
	end
end

-----------------------------------------

function Atr_ShowElems (tt)

	for i,x in ipairs(tt) do
		x:Show();
	end
end

-----------------------------------------

function Atr_OnAuctionUpdate ()

	if (not Atr_IsTabSelected()) then
		Atr_ClearScanCache();		-- if not our tab, we have no idea what happened so must flush all caches
		return;
	end;

	if (gActionState == ATR_BUYING) then
		gActionState = ATR_NOTHING;
		Atr_Continue_Buying ();
		return;
	elseif (gActionState == ATR_JUST_BOUGHT) then
		gActionState = ATR_NOTHING;
		Atr_AdjustScanForBoughtItem();
		return;
	end


	if (gCurrentPane.activeScan.processing_state == KM_POSTQUERY) then

		local done = gCurrentPane.activeScan:AnalyzeResultsPage();

		if (done) then
			
			if (Atr_IsModeCreateAuction() and #gCurrentPane.activeScan.scanData == 0) then
				gCurrentPane.activeScan:AddExternalDataToScan();
			end
			
			gCurrentPane.activeScan:Finish();

			Atr_Process_Historydata ();
			
			Atr_FindBestHistoricalAuction ();
			Atr_FindBestCurrentAuction ();
			gCurrentPane:SetToShowCurrent();
			
			if (Atr_IsModeBuy()) then
				Atr_Shop_OnFinishScan ();
			end
			
			gCurrentPane.UINeedsUpdate = true;

		end
	end

end

-----------------------------------------

function Atr_ClearTop ()
	Atr_HideElems (recommendElements);
	Atr_HideElems (overallElements);

	AuctionatorMessageFrame:Hide();
	AuctionatorMessage2Frame:Hide();
end

-----------------------------------------

function Atr_ClearAll ()
	Atr_ClearTop();
	Atr_ClearList();
end

-----------------------------------------

function Atr_SetMessage (msg)
	Atr_HideElems (recommendElements);
	Atr_HideElems (overallElements);

	if (gCurrentPane.itemLink) then
		Atr_Recommend_Text:Show ();
		Atr_Recommend_Text:SetText (gCurrentPane.itemName);
		Atr_RecommendItem_Tex:Show();
		Atr_SetTextureButton ("Atr_RecommendItem_Tex", 1);

		AuctionatorMessage2Frame:SetText (msg);
		AuctionatorMessage2Frame:Show();
		AuctionatorMessageFrame:Hide();
	else
		AuctionatorMessageFrame:SetText (msg);
		AuctionatorMessageFrame:Show();
		AuctionatorMessage2Frame:Hide();
	end
end



-----------------------------------------

function Atr_SortHistoryData (x, y)

	return x.when > y.when;

end

-----------------------------------------

function BuildHtag (type, y, m, d)

	local t = time({year=y, month=m, day=d, hour=0});

	return tostring (ToTightTime(t))..":"..type;
end

-----------------------------------------

function ParseHtag (tag)
	local when, type = strsplit(":", tag);

	if (type == nil) then
		type = "hx";
	end

	when = FromTightTime (tonumber (when));

	return when, type;
end

-----------------------------------------

function ParseHist (tag, hist)

	local when, type = ParseHtag(tag);

	local price, count	= strsplit(":", hist);

	price = tonumber (price);

	local stacksize, numauctions;

	if (type == "hx") then
		stacksize	= tonumber (count);
		numauctions	= 1;
	else
		stacksize = 0;
		numauctions	= tonumber (count);
	end

	return when, type, price, stacksize, numauctions;

end

-----------------------------------------

function CalcAbsTimes (when, whent)

	local absYear	= whent.year - 2000;
	local absMonth	= (absYear * 12) + whent.month;
	local absDay	= floor ((when - gTimeZero) / (60*60*24));

	return absYear, absMonth, absDay;

end

-----------------------------------------

function Atr_Condense_History (itemname)

	if (AUCTIONATOR_PRICING_HISTORY[itemname] == nil) then
		return;
	end

	local tempHistory = {};

	local now			= time();
	local nowt			= date("*t", now);

	local absNowYear, absNowMonth, absNowDay = CalcAbsTimes (now, nowt);

	local n = 1;
	local tag, hist, newtag, stacksize, numauctions;
	for tag, hist in pairs (AUCTIONATOR_PRICING_HISTORY[itemname]) do
		if (tag ~= "is") then

			local when, type, price, stacksize, numauctions = ParseHist (tag, hist);

			local whnt = date("*t", when);

			local absYear, absMonth, absDay	= CalcAbsTimes (when, whnt);

			if (absNowYear - absYear >= 3) then
				newtag = BuildHtag ("hy", whnt.year, 1, 1);
			elseif (absNowMonth - absMonth >= 2) then
				newtag = BuildHtag ("hm", whnt.year, whnt.month, 1);
			elseif (absNowDay - absDay >= 2) then
				newtag = BuildHtag ("hd", whnt.year, whnt.month, whnt.day);
			else
				newtag = tag;
			end

			tempHistory[n] = {};
			tempHistory[n].price		= price;
			tempHistory[n].numauctions	= numauctions;
			tempHistory[n].stacksize	= stacksize;
			tempHistory[n].when			= when;
			tempHistory[n].newtag		= newtag;
			n = n + 1;
		end
	end

	-- clear all the existing history

	local is = AUCTIONATOR_PRICING_HISTORY[itemname]["is"];

	AUCTIONATOR_PRICING_HISTORY[itemname] = {};
	AUCTIONATOR_PRICING_HISTORY[itemname]["is"] = is;

	-- repopulate the history

	local x;

	for x = 1,#tempHistory do

		local thist		= tempHistory[x];
		local newtag	= thist.newtag;

		if (AUCTIONATOR_PRICING_HISTORY[itemname][newtag] == nil) then

			local when, type = ParseHtag (newtag);

			local count = thist.numauctions;
			if (type == "hx") then
				count = thist.stacksize;
			end

			AUCTIONATOR_PRICING_HISTORY[itemname][newtag] = tostring(thist.price)..":"..tostring(count);

		else

			local hist = AUCTIONATOR_PRICING_HISTORY[itemname][newtag];

			local when, type, price, stacksize, numauctions = ParseHist (newtag, hist);

			local newNumAuctions = numauctions + thist.numauctions;
			local newPrice		 = ((price * numauctions) + (thist.price * thist.numauctions)) / newNumAuctions;

			AUCTIONATOR_PRICING_HISTORY[itemname][newtag] = tostring(newPrice)..":"..tostring(newNumAuctions);
		end
	end

end

-----------------------------------------

function Atr_Process_Historydata ()

	-- Condense the data if needed - only once per session for each item

	if (gCondensedThisSession[gCurrentPane.itemName] == nil) then

		gCondensedThisSession[gCurrentPane.itemName] = true;

		Atr_Condense_History(gCurrentPane.itemName);
	end


	-- build the sorted history list

	gCurrentPane.sortedHist = {};

	if (AUCTIONATOR_PRICING_HISTORY[gCurrentPane.itemName]) then
		local n = 1;
		local tag, hist;
		for tag, hist in pairs (AUCTIONATOR_PRICING_HISTORY[gCurrentPane.itemName]) do
			if (tag ~= "is") then
				local when, type, price, stacksize, numauctions = ParseHist (tag, hist);

				gCurrentPane.sortedHist[n]				= {};
				gCurrentPane.sortedHist[n].itemPrice	= price;
				gCurrentPane.sortedHist[n].buyoutPrice	= price;
				gCurrentPane.sortedHist[n].stackSize	= stacksize;
				gCurrentPane.sortedHist[n].when			= when;
				gCurrentPane.sortedHist[n].yours		= true;
				gCurrentPane.sortedHist[n].type			= type;

				if (stacksize == 0) then
					gCurrentPane.sortedHist[n].stackSize = numauctions;
				end

				n = n + 1;
			end
		end
	end

	table.sort (gCurrentPane.sortedHist, Atr_SortHistoryData);

	Atr_SetRBcolor ("Atr_History_RBText", #gCurrentPane.sortedHist);

end


-----------------------------------------

function Atr_SetRBcolor (rbname, count)

	local text = getglobal(rbname);

	if (count == 0) then
		text:SetTextColor(0.7, 0.7, 0.3);
	else
		text:SetTextColor(1, 1, 0);
	end
end

-----------------------------------------

function Atr_ShowingCurrentAuctions ()
	if (gCurrentPane) then
		return gCurrentPane:ShowCurrent();
	end
	
	return true;
end

-----------------------------------------

function Atr_UpdateRecommendation ()

	local basedata;

	if (Atr_ShowingCurrentAuctions()) then
		if (not gCurrentPane.currIndex) then
			if (gCurrentPane.activeScan.numMatches == 0 and #gCurrentPane.activeScan.sortedData == 0) then
				Atr_SetMessage ("No current auctions found");
			elseif (gCurrentPane.activeScan.numMatches == 0) then
				Atr_SetMessage ("No current auctions found\n\n(related auctions shown)");
			elseif (gCurrentPane.activeScan.numMatchesWithBuyout == 0) then
				Atr_SetMessage ("No current auctions with buyouts found");
			else
				Atr_SetMessage ("");
			end
			return;
		end

		basedata = gCurrentPane.activeScan.sortedData[gCurrentPane.currIndex];
	else
		if (not gCurrentPane.histIndex) then
			Atr_SetMessage ("Auctionator has yet to record any auctions for this item");
			return;
		end

		basedata = gCurrentPane.sortedHist[gCurrentPane.histIndex];
	end

	if (not Atr_IsModeCreateAuction()) then
		Atr_HideElems (recommendElements);
		AuctionatorMessageFrame:Hide();
		AuctionatorMessage2Frame:Hide();

		Atr_Recommend_Text:Show ();
		Atr_Recommend_Text:SetText (gCurrentPane.itemName);
		Atr_RecommendItem_Tex:Show();

		Atr_SetTextureButton ("Atr_RecommendItem_Tex", 1);
		return;
	end

	if (gCurrentPane.stackSize == 0) then
		return;
	end

	local newBuyoutPrice = math.floor (basedata.itemPrice * gCurrentPane.stackSize);

	if (not basedata.yours) then
		newBuyoutPrice = Atr_CalcNewPrice (newBuyoutPrice);
	end

	local discount = 1.00 - (AUCTIONATOR_SAVEDVARS.STARTING_DISCOUNT / 100);

	local newStartPrice = Atr_CalcNewPrice(math.floor(newBuyoutPrice * discount));

	if (AUCTIONATOR_SAVEDVARS.STARTING_DISCOUNT == 0) then		-- zero means zero
		newStartPrice = newBuyoutPrice;
	end
	
	Atr_ShowElems (recommendElements);
	AuctionatorMessageFrame:Hide();
	AuctionatorMessage2Frame:Hide();

	Atr_Recommend_Text:SetText ("Recommended Buyout Price");
	Atr_RecommendPerStack_Text:SetText ("for your stack of "..gCurrentPane.stackSize);

	Atr_SetTextureButton ("Atr_RecommendItem_Tex", gCurrentPane.stackSize);


	MoneyFrame_Update ("Atr_RecommendPerItem_Price",  ZC_round(newBuyoutPrice / gCurrentPane.stackSize));
	MoneyFrame_Update ("Atr_RecommendPerStack_Price", ZC_round(newBuyoutPrice));

	MoneyInputFrame_SetCopper (BuyoutPrice, newBuyoutPrice);
	MoneyInputFrame_SetCopper (StartPrice,  newStartPrice);

	Atr_EnableDisableBatchCreate();

	local cheapestStack = gCurrentPane.activeScan.bestPrices[gCurrentPane.stackSize];

	Atr_Recommend_Basis_Text:SetTextColor (1,1,1);

	if (basedata.type == "e") then
		Atr_Recommend_Basis_Text:SetText ("(based on Wowecon price)");
		Atr_Recommend_Basis_Text:SetTextColor (0.6, 0.6, 1.0);
	elseif (basedata.type == "h") then
		Atr_Recommend_Basis_Text:SetText ("(based on GoingPrice - Wowhead)");
		Atr_Recommend_Basis_Text:SetTextColor (0.6, 0.6, 1.0);
	elseif (basedata.type == "k") then
		Atr_Recommend_Basis_Text:SetText ("(based on GoingPrice - Allakhazam)");
		Atr_Recommend_Basis_Text:SetTextColor (0.6, 0.6, 1.0);
	elseif (gCurrentPane.activeScan.absoluteBest and basedata.stackSize == gCurrentPane.activeScan.absoluteBest.stackSize and basedata.buyoutPrice == gCurrentPane.activeScan.absoluteBest.buyoutPrice) then
		Atr_Recommend_Basis_Text:SetText ("(based on cheapest current auction)");
	elseif (cheapestStack and basedata.stackSize == cheapestStack.stackSize and basedata.buyoutPrice == cheapestStack.buyoutPrice) then
		Atr_Recommend_Basis_Text:SetText ("(based on cheapest stack of the same size)");
	else
		Atr_Recommend_Basis_Text:SetText ("(based on selected auction)");
	end

end

-----------------------------------------

function Atr_EnableDisableBatchCreate()

	if (gCurrentPane.hasMultiples and MoneyInputFrame_GetCopper(StartPrice) > 0) then
		Atr_BatchButton:Enable();
	else
		Atr_BatchButton:Disable();
	end

end

-----------------------------------------

function Atr_Find_More (pItemName, pItemCount)

	if (pItemName == nil) then
		return false;
	end

	local bagID, slotID, numslots;
	local foundSoFar = 0;

	for bagID = kFirstBag, kLastBag do
		numslots = GetContainerNumSlots (bagID);
		for slotID = 1,numslots do
			local itemLink = GetContainerItemLink(bagID, slotID);
			if (itemLink) then
				local itemName				= GetItemInfo(itemLink);
				local texture, itemCount	= GetContainerItemInfo(bagID, slotID);

				if (itemName == pItemName) then
					foundSoFar = foundSoFar + itemCount;
					if (foundSoFar >= pItemCount * 2) then
						return true;
					end
				end
			end
		end
	end

	return false;

end

-----------------------------------------

function Atr_SetTextureButton(elementName, count, itemlink)

	if (not itemlink) then
		itemlink = gCurrentPane.itemLink;
		if (itemlink == nil) then
			itemlink = gCurrentPane.activeScan.itemLink;
		end
	end

	texture = GetItemIcon (itemlink);

	local textureElement = getglobal (elementName);
	local countElement   = getglobal (elementName.."Count");

	if (texture) then
		textureElement:SetNormalTexture (texture);
		if (count > 1) then
			countElement:SetText (count);
			countElement:Show();
		else
			countElement:Hide();
		end
	else
		textureElement:Hide();
	end

end

-----------------------------------------

function Atr_SetRecTooltip ()
	if (gCurrentPane.itemLink) then
		GameTooltip:SetHyperlink (gCurrentPane.itemLink);
	elseif (gCurrentPane.activeScan.itemLink) then
		GameTooltip:SetHyperlink (gCurrentPane.activeScan.itemLink);
	elseif (gJustPosted_ItemLink) then
		GameTooltip:SetHyperlink (gJustPosted_ItemLink);
	end
end

-----------------------------------------

function Atr_OnAuctionHouseShow()

	gOpenAllBags = AUCTIONATOR_OPEN_ALL_BAGS;

	if (AUCTIONATOR_OPEN_FIRST ~= 0) then
		AuctionFrameTab_OnClick (nil, Atr_FindTabIndex(SELL_TAB));
	end

	if (AUCTIONATOR_OPEN_BUY ~= 0) then
		AuctionFrameTab_OnClick (nil, Atr_FindTabIndex(BUY_TAB));
	end

	Atr_ResetDuration();

	if (gJustPosted_ItemName) then
		gJustPosted_ItemName = nil;
		gSellPane:SetItem ("", nil);
	end
end

-----------------------------------------

function Atr_OnAuctionHouseClosed()

	Atr_HideAllDialogs();
	gAutoSelling = false;

	Atr_ClearScanCache();

	gSellPane:SetItem ("");
	gShopPane:SetItem ("");
	gMorePane:SetItem ("");

--	UpdateAddOnMemoryUsage();
--	local mem = GetAddOnMemoryUsage("Auctionator");
--	ZC_redmsg ("Memory:  "..math.floor(mem).." KB");
	
end

-----------------------------------------

function Atr_HideAllDialogs()

	AuctionatorOptionsFrame:Hide();
	AuctionatorDescriptionFrame:Hide();
	AuctionatorUndercuttingConfig:Hide();
	AuctionatorBasicOptionsFrame:Hide();
	Atr_Batch_Frame:Hide();
	Atr_Error_Frame:Hide();
	Atr_Mask:Hide();

end



-----------------------------------------

function Atr_SelectSellTab()

	AuctionFrameTab_OnClick (nil, Atr_FindTabIndex(SELL_TAB));

end

-----------------------------------------

function Atr_BasicOptionsUpdate(self, elapsed)

	self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed;

	if (self.TimeSinceLastUpdate > 0.25) then

		self.TimeSinceLastUpdate = 0;

		if (AuctionatorOption_Def_Duration_CB:GetChecked()) then
			AuctionatorOption_Durations:Show();
		else
			AuctionatorOption_Durations:Hide();
		end

	end
end

-----------------------------------------

function Atr_OnUpdate(self, elapsed)

	if (Atr_IsTabSelected()) then
		Atr_Idle (self, elapsed);
	end

end


-----------------------------------------

function Atr_Idle(self, elapsed)

	self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed;

	if (AuctionatorMessageFrame == nil) then
		return;
	end

	if (self.TimeSinceLastUpdate > 0.25) then

		self.TimeSinceLastUpdate = 0;

		if (gHentryTryAgain) then
			Atr_HEntryOnClick();
			return;
		end

		if (gCurrentPane.activeScan.processing_state == KM_PREQUERY) then		------- check whether to send a new auction query to get the next page -------
			gCurrentPane.activeScan:Continue();
		end

		if (gAutoSelling) then
			Atr_Idle_AutoSelling (self);
		end

		AuctionatorUpdateUI ();

		if (gHideAPFrameCheck == nil) then	-- for the addon 'Auction Profit' (flags for efficiency so we only check one time)
			gHideAPFrameCheck = true;
			if (AP_Bid_MoneyFrame) then	
				AP_Bid_MoneyFrame:Hide();
				AP_Buy_MoneyFrame:Hide();
			end
		end

	end

	if (Atr_IsTabSelected(SELL_TAB)) then
		Atr_EnableDisableBatchCreate();
		Atr_Idle_CheckForAutoSplit ();
		Atr_Idle_CheckForNewAuctionItem (self);		------- check whether the "sell" item has changed -------
	end
end

---------------------------------------------------------

local gPrevSellItemLink;

---------------------------------------------------------

function Atr_Idle_CheckForNewAuctionItem (self)

	if (gAutoSelling) then
		return;
	end

	local auctionItemName, auctionCount, auctionLink = Atr_GetSellItemInfo();

	if (gPrevSellItemLink ~= auctionLink) then

		gPrevSellItemLink = auctionLink;
		
		if (auctionLink) then
			gJustPosted_ItemName = nil;
			Atr_ClearList();		-- better UE
			gSellPane:SetToShowCurrent();
			Atr_Batch_Frame:Hide();
		end
		
		MoneyInputFrame_SetCopper (BuyoutPrice, 0);
		MoneyInputFrame_SetCopper (StartPrice,  0);
		
		if (gJustPosted_ItemName == nil) then
			gSellPane:SetItem (auctionItemName, auctionLink, auctionCount, 20);
		end
		
--		if (auctionLink == nil and gJustPosted_ItemName == nil) then
--			gCurrentPane.activeScan = Atr_FindScan (nil);
--			Atr_ClearList();
--		end
	
	elseif (gSellPane.stackSize ~= auctionCount) then
	
		gSellPane.stackSize = auctionCount;
	
		Atr_FindBestCurrentAuction();
		
		gSellPane.UINeedsUpdate = true;
		
	end

end

---------------------------------------------------------

function AuctionatorUpdateUI ()

	if (gCurrentPane.UINeedsUpdate) then
	
		gCurrentPane.UINeedsUpdate = false;
		
		Atr_CurrentAuctions_RB:SetChecked(nil);
		Atr_History_RB:SetChecked(nil);
		
		if (gCurrentPane:ShowCurrent()) then
			Atr_CurrentAuctions_RB:SetChecked();
			Atr_ShowCurrentAuctions();
		else
			Atr_History_RB:SetChecked();
			Atr_ShowHistory();
		end
		
		if (Atr_IsTabSelected(SELL_TAB) and gJustPosted_ItemName ~= nil) then

			Atr_Recommend_Text:SetText ("Auction created for "..gJustPosted_ItemName);
			MoneyFrame_Update ("Atr_RecommendPerStack_Price", gJustPosted_BuyoutPrice);
			Atr_SetTextureButton ("Atr_RecommendItem_Tex", gJustPosted_StackSize, gJustPosted_ItemLink);

			gCurrentPane.currIndex = gCurrentPane.activeScan:FindInSortedData (gJustPosted_StackSize, gJustPosted_BuyoutPrice);

			Atr_HighlightEntry (gCurrentPane.currIndex);		-- highlight the newly created auction(s)
		
		elseif (gCurrentPane.itemName == "") then
			if (Atr_IsModeCreateAuction()) then
				Atr_SetMessage ("Drag an item you want to sell to this area.");
			elseif (Atr_IsModeBuy()) then
				Atr_SetMessage ("Select an item from the list on the left\n or type the name of an item to start a scan.\n\nItem names much match exactly.");
			else
				Atr_SetMessage ("");
			end
		else
			Atr_UpdateRecommendation ();
		end
		
		if (Atr_IsTabSelected(SELL_TAB)) then
		
			gCurrentPane.hasMultiples = Atr_Find_More (gCurrentPane.itemName, gCurrentPane.stackSize);

			Atr_EnableDisableBatchCreate();

			if (gAutoSplitAndBatch) then
				gAutoSplitAndBatch = false;
				
				if (Atr_BatchButton:IsEnabled() == 1) then
					Atr_ShowBatchFrame();
				end
			end
		end

		if (Atr_IsTabSelected(BUY_TAB)) then
			Atr_Shop_UpdateUI();
		end

	end

end

-----------------------------------------

function Atr_SetActionNormal()

	gActionState = ATR_NORMAL_SCAN;

end

-----------------------------------------

function BS_GetCount(bs)

	local texture, count = GetContainerItemInfo (bs.bagID, bs.slotID);
	if (texture ~= nil) then
		return count;
	end

	return 0;

end

-----------------------------------------

function BS_InCslots(xbs)

	local i, bs;

	for i,bs in pairs(cslots) do
		if (xbs.bagID == bs.bagID and xbs.slotID == bs.slotID) then
			return true;
		end
	end

	return false;

end

-----------------------------------------

function BS_GetEmptySlot()


	if (gEmptyBScached == nil or BS_GetCount (gEmptyBScached) ~= 0) then

		gEmptyBScached = nil;

		for bagID = kFirstBag, kLastBag do
			numslots = GetContainerNumSlots (bagID);
			for slotID = 1,numslots do
				local itemLink = GetContainerItemLink(bagID, slotID);
				if (itemLink == nil) then
					gEmptyBScached = {};
					gEmptyBScached.bagID  = bagID;
					gEmptyBScached.slotID = slotID;

					-- add to cslots if not already there

					if (not BS_InCslots (gEmptyBScached)) then
--						ZC_chatmsg ("Inserting "..bagID.."/"..slotID);
						tinsert (cslots, gEmptyBScached);
					end

					return gEmptyBScached;
				end
			end
		end
	end

	return gEmptyBScached;
end



-----------------------------------------

function BS_PostAuction(bs)

	PickupContainerItem (bs.bagID, bs.slotID);

	local infoType = GetCursorInfo()

	if (infoType == "item") then
		ClickAuctionSellItemButton();
		ClearCursor();
	end

	StartAuction (gAutoSell_StartPrice, gAutoSell_BuyoutPrice, gAutoSell_Hours * 60);

end

-----------------------------------------

function BS_FindGoodStack()

	local dstr = "";
	local i;

	for i, bs in pairs (cslots) do

		dstr = dstr..BS_GetCount (bs).." ";

		if (BS_GetCount (bs) == gAutoSell_GoodStackSize) then
--			ZC_chatmsg ("FindGood: "..dstr);
			return bs
		end

	end

--	ZC_chatmsg ("FindGoodx: "..dstr);

	return nil;

end

-----------------------------------------

function BS_MergeSmallStacks()			-- find the 2 smallest stacks and merge them together if possible

	if (#cslots < 2) then
		return false;
	end

	local i, bs;

	local zbs	= nil;		-- smallest
	local ybs	= nil;		-- second smallest

	local zcount = 10000;
	local ycount = 10000;

	for i, bs in pairs (cslots) do
		local count = BS_GetCount (bs);

		if (count > 0) then
			if (count < zcount) then
				ybs = zbs;	ycount = zcount;
				zbs = bs;	zcount = count;
			elseif (count < ycount) then
				ybs = bs;	ycount = count;
			end
		end
	end

	if (zcount == 10000 or ycount == 10000) then
		return false;
	end

	-- try to make a "good" stack

	if (zcount < gAutoSell_GoodStackSize and ycount + zcount >= gAutoSell_GoodStackSize) then
		SplitContainerItem  (ybs.bagID, ybs.slotID, gAutoSell_GoodStackSize - zcount);
		PickupContainerItem (zbs.bagID, zbs.slotID);

		gAutoSell_targetBS		= zbs;
		gAutoSell_targetCount	= gAutoSell_GoodStackSize;

		return true;

	end

	-- merge them best as possible

	local numToMove = zcount;
	if (zcount + ycount > gAutoSell_FullStackSize) then
		numToMove = gAutoSell_FullStackSize - ycount;
	end

	if (numToMove > 0) then
		SplitContainerItem  (zbs.bagID, zbs.slotID, numToMove);
		PickupContainerItem (ybs.bagID, ybs.slotID);

		gAutoSell_targetBS		= ybs;
		gAutoSell_targetCount	= ycount + numToMove;

		return true;
	end

	return false;

end


-----------------------------------------

function BS_SplitLargeStack()

	local i, bs;

	local emptyBS = BS_GetEmptySlot ();

	for i, bs in pairs (cslots) do
		local count = BS_GetCount (bs);

		if (count > gAutoSell_GoodStackSize) then
			if (emptyBS) then
				SplitContainerItem  (bs.bagID, bs.slotID, gAutoSell_GoodStackSize);
				PickupContainerItem (emptyBS.bagID, emptyBS.slotID);

				gAutoSell_targetBS		= emptyBS;
				gAutoSell_targetCount	= gAutoSell_GoodStackSize;
				return true;
			end
		end

	end

	return false;

end

-----------------------------------------

function Atr_Idle_AutoSelling()

	if (CursorHasItem()) then
		return;
	end

	if (gAutoSellState == AUCTION_POST_PENDING) then
		return;
	end

	if (gAutoSellState == STACK_MERGE_PENDING or gAutoSellState == STACK_SPLIT_PENDING) then

		if (BS_GetCount (gAutoSell_targetBS) == gAutoSell_targetCount) then
			ClearCursor();
			gAutoSellState = SETTLED;
		else
			return;
		end
	end

	if (gAutoSellState ~= SETTLED) then
		return
	end

	-- let's see if we're done

	if (gAutoSell_AuctionNum > gAutoSell_NumAuctionsToCreate) then
		gAutoSelling = false;
		return;
	end


	-- if there's a stack that's ready to sell, sell it

	local goodBS = BS_FindGoodStack();

	if (goodBS) then
		BS_PostAuction (goodBS);
		gAutoSellState = AUCTION_POST_PENDING;
		return;
	end

	-- see if we can split a larger stack to get a sellable stack

	local success = BS_SplitLargeStack();
	if (success) then
		gAutoSellState = STACK_SPLIT_PENDING;
		return;
	end

	-- see if we can merge two smaller stacks

	local success = BS_MergeSmallStacks();
	if (success) then
		gAutoSellState = STACK_MERGE_PENDING;
		return;
	end


	-- nothing left to do - we're done

	gAutoSelling = false;

end


-----------------------------------------

function Atr_GetActiveAuction (n)

	if (#gActiveAuctions == 0) then

		local foundnames = {};

		local k = 1;
		local i = 1;
		while (true) do
			local name, texture, count = GetAuctionItemInfo ("owner", i);
			if (name == nil) then
				break;
			end

			if (count > 0) then		-- count is 0 for sold items
				if (foundnames[name] == nil) then
					gActiveAuctions[k] = name;
					k = k + 1;
				end

				foundnames[name] = 1;
			end

			i = i + 1;
		end
	end


	if (#gActiveAuctions == 0) then
		return nil;
	end

	return gActiveAuctions[n];


end

-----------------------------------------

function Atr_DisplayHlist ()

	if (Atr_IsTabSelected (BUY_TAB)) then		-- done this way because OnScrollFrame always calls Atr_DisplayHlist
		Atr_DisplaySlist();
		return;
	end

	local doFull = (UIDropDownMenu_GetSelectedValue(Atr_DropDown1) == MODE_LIST_ALL);
	
	Atr_BuildGlobalHistoryList (doFull);
	
	local numrows = #gHistoryItemList;

	local line;							-- 1 through NN of our window to scroll
	local dataOffset;					-- an index into our data calculated from the scroll offset

	FauxScrollFrame_Update (Atr_Hlist_ScrollFrame, numrows, ITEM_HIST_NUM_LINES, 16);

	for line = 1,ITEM_HIST_NUM_LINES do

		gCurrentPane.hlistScrollOffset = FauxScrollFrame_GetOffset (Atr_Hlist_ScrollFrame);
		
		dataOffset = line + gCurrentPane.hlistScrollOffset;

		local lineEntry = getglobal ("AuctionatorHEntry"..line);

		lineEntry:SetID(dataOffset);

		if (dataOffset <= numrows and gHistoryItemList[dataOffset]) then

			local lineEntry_text = getglobal("AuctionatorHEntry"..line.."_EntryText");

			local iName = gHistoryItemList[dataOffset];

			lineEntry_text:SetText	(Atr_AbbrevItemName (iName));

			if (iName == gCurrentPane.itemName) then
				lineEntry:SetButtonState ("PUSHED", true);
			else
				lineEntry:SetButtonState ("NORMAL", false);
			end

			lineEntry:Show();
		else
			lineEntry:Hide();
		end
	end

end

-----------------------------------------

function Atr_ClearHlist ()
	local line;
	for line = 1,ITEM_HIST_NUM_LINES do
		local lineEntry = getglobal ("AuctionatorHEntry"..line);
		lineEntry:Hide();
		
		local lineEntry_text = getglobal("AuctionatorHEntry"..line.."_EntryText");
		lineEntry_text:SetText		("");
		lineEntry_text:SetTextColor	(.6,.6,.6);
	end

end

-----------------------------------------

function Atr_HEntryOnClick()

	if (gCurrentPane == gShopPane) then
		Atr_SEntryOnClick();
		return;
	end

	local line = this;

	if (gHentryTryAgain) then
		line = gHentryTryAgain;
		gHentryTryAgain = nil;
	end

	local _, itemLink;
	local entryIndex = line:GetID();
	local itemName	 = gHistoryItemList[entryIndex];

	if (AUCTIONATOR_PRICING_HISTORY[itemName]) then
		local itemId, suffixId, uniqueId = strsplit(":", AUCTIONATOR_PRICING_HISTORY[itemName]["is"])

		local itemId	= tonumber(itemId);

		if (suffixId == nil) then	suffixId = 0;
		else		 				suffixId = tonumber(suffixId);
		end

		if (uniqueId == nil) then	uniqueId = 0;
		else		 				uniqueId = tonumber(suffixId);
		end

		local itemString = "item:"..itemId..":0:0:0:0:0:"..suffixId..":"..uniqueId;

		_, itemLink, _, _, _, _, _, _ = GetItemInfo(itemString);

		if (itemLink == nil) then		-- pull it into the cache and go back to the idle loop to wait for it to appear
			MyScanningTooltip:SetHyperlink(itemString);
			gHentryTryAgain = line;
	--		ZC_chatmsg ("pulling "..itemName.." into the local cache");
			return;
		end
	end
	
	gCurrentPane.UINeedsUpdate = true;
	
	gCurrentPane:SetItem (itemName, itemLink, 0, 60);

	Atr_FindBestCurrentAuction ();

	Atr_Process_Historydata ();

	Atr_DisplayHlist();	 -- for the highlight

	PlaySound ("igMainMenuOptionCheckBoxOn");
end

-----------------------------------------

function Atr_ShowWhichRB (id)

	if (gCurrentPane.activeScan.processing_state ~= KM_NULL_STATE) then		-- if we're scanning auctions don't respond
		return;
	end

	PlaySound("igMainMenuOptionCheckBoxOn");

	if (id == 1) then
		gCurrentPane:SetToShowCurrent();
	else
		gCurrentPane:SetToShowHistory();
	end
	
	gCurrentPane.UINeedsUpdate = true;

end


-----------------------------------------

function Atr_RedisplayAuctions ()
	if (Atr_ShowingCurrentAuctions()) then
		Atr_ShowCurrentAuctions();
	else
		Atr_ShowHistory();
	end
end

-----------------------------------------

function Atr_BuildHistItemText(data)

	local stacktext = "";
--	if (data.stackSize > 1) then
--		stacktext = " (stack of "..data.stackSize..")";
--	end

	local now		= time();
	local nowtime	= date ("*t");

	local when		= data.when;
	local whentime	= date ("*t", when);

	local numauctions = data.stackSize;

	local datestr = "";

	if (data.type == "hy") then
		return "average of your auctions for "..whentime.year;
	elseif (data.type == "hm") then
		if (nowtime.year == whentime.year) then
			return "average of your auctions for "..date("%B", when);
		else
			return "average of your auctions for "..date("%B %Y", when);
		end
	elseif (data.type == "hd") then
		return "average of your auctions for "..monthDay(whentime);
	else
		return "your auction on "..monthDay(whentime)..date(" at %I:%M %p", when);
	end
end

-----------------------------------------

function monthDay (when)

	local t = time(when);

	local s = date("%b ", t);

	return s..when.day;

end

-----------------------------------------

function Atr_ClearList ()

	Atr_Col1_Heading:Hide();
	Atr_Col3_Heading:Hide();
	Atr_Col4_Heading:Hide();

	local line;							-- 1 through 12 of our window to scroll

	FauxScrollFrame_Update (AuctionatorScrollFrame, 0, 12, 16);

	for line = 1,12 do
		local lineEntry = getglobal ("AuctionatorEntry"..line);
		lineEntry:Hide();
	end

end


-----------------------------------------

function Atr_SetMFcolor (frameName, blue)

	local goldButton = getglobal(frameName.."GoldButton");
	local silverButton = getglobal(frameName.."SilverButton");
	local copperButton = getglobal(frameName.."CopperButton");

	if (blue) then
		goldButton:SetNormalFontObject(NumberFontNormalRightATRblue);
		silverButton:SetNormalFontObject(NumberFontNormalRightATRblue);
		copperButton:SetNormalFontObject(NumberFontNormalRightATRblue);
	else
		goldButton:SetNormalFontObject(NumberFontNormalRight);
		silverButton:SetNormalFontObject(NumberFontNormalRight);
		copperButton:SetNormalFontObject(NumberFontNormalRight);
	end
	
end

-----------------------------------------

function Atr_ShowCurrentAuctions()

	Atr_Col1_Heading:Hide();
	Atr_Col3_Heading:Hide();
	Atr_Col4_Heading:Hide();

	local numrows = #gCurrentPane.activeScan.sortedData;

	if (numrows > 0) then
		Atr_Col1_Heading:Show();
		Atr_Col3_Heading:Show();
		Atr_Col4_Heading:Show();
	end

	Atr_Col3_Heading:SetText ("Current Auctions");

	if (gCurrentPane.activeScan.hasStack) then
		Atr_Col4_Heading:SetText ("Total Price");
	else
		Atr_Col4_Heading:SetText ("");
	end

	local line		 = 0;															-- 1 through 12 of our window to scroll
	local dataOffset = FauxScrollFrame_GetOffset (AuctionatorScrollFrame);			-- an index into our data calculated from the scroll offset

	FauxScrollFrame_Update (AuctionatorScrollFrame, numrows, 12, 16);

	while (line < 12) do

		dataOffset	= dataOffset + 1;
		line		= line + 1;

		local lineEntry = getglobal ("AuctionatorEntry"..line);

		lineEntry:SetID(dataOffset);

		if (dataOffset > numrows or not gCurrentPane.activeScan.sortedData[dataOffset]) then

			lineEntry:Hide();

		else
			local data = gCurrentPane.activeScan.sortedData[dataOffset];

			local lineEntry_item_tag = "AuctionatorEntry"..line.."_PerItem_Price";

			local lineEntry_item		= getglobal(lineEntry_item_tag);
			local lineEntry_itemtext	= getglobal("AuctionatorEntry"..line.."_PerItem_Text");
			local lineEntry_text		= getglobal("AuctionatorEntry"..line.."_EntryText");
			local lineEntry_stack		= getglobal("AuctionatorEntry"..line.."_StackPrice");

			lineEntry_itemtext:SetText	("");
			lineEntry_text:SetText	("");
			lineEntry_stack:SetText	("");

			Atr_SetMFcolor (lineEntry_item_tag);
			
			local entrytext = "";

			if (data.type == "n" or data.type == "e" or data.type == "h" or data.type == "k") then

				lineEntry:Show();

				if (gCurrentPane.activeScan.itemSoftName) then

					entrytext = gCurrentPane.activeScan.itemSoftName.." "..data.suffix;

					if ( entrytext == gCurrentPane.activeScan.itemName ) then	lineEntry_text:SetTextColor (1.0, 1.0, 1.0);
					else														lineEntry_text:SetTextColor (0.5, 0.5, 0.5);
					end
				else
					entrytext = string.format ("%i %s of %i", data.count, ZC_pluralizeIf ("stack", data.count), data.stackSize);

					if ( data.stackSize == gCurrentPane.stackSize or gCurrentPane.stackSize == 0) then	lineEntry_text:SetTextColor (1.0, 1.0, 1.0);
					else																				lineEntry_text:SetTextColor (0.6, 0.6, 0.6);
					end;
				end

				if (data.yours) then
					 entrytext = entrytext.." (yours)";
				end

				if (data.type == "e" or data.type == "h" or data.type == "k") then
				
					entrytext = "Wowecon price";
					if (data.type == "h") then entrytext = "GoingPrice - Wowhead"; end;
					if (data.type == "k") then entrytext = "GoingPrice - Allakhazam"; end;
				
					if (gCurrentPane.activeScan.itemSoftName) then
						entrytext = gCurrentPane.activeScan.itemName.." ("..entrytext..")";
					end
					
					Atr_SetMFcolor (lineEntry_item_tag, true);
					lineEntry_text:SetTextColor (0.6, 0.6, 1.0);
				end
					
				lineEntry_text:SetText (entrytext);

				if (data.buyoutPrice == 0) then
					lineEntry_item:Hide();
					lineEntry_itemtext:Show();
					lineEntry_itemtext:SetText ("no buyout price");
				else
					lineEntry_item:Show();
					lineEntry_itemtext:Hide();
					MoneyFrame_Update (lineEntry_item_tag, ZC_round(data.buyoutPrice/data.stackSize) );

					if (gCurrentPane.activeScan.itemSoftName == nil and data.stackSize > 1) then
						lineEntry_stack:SetText (ZC_priceToString(data.buyoutPrice));
						lineEntry_stack:SetTextColor (0.6, 0.6, 0.6);
					end
				end
			
			else
				ZC_redmsg ("Unknown datatype:");
				ZC_redmsg (data.type);
			end
		end
	end

	Atr_HighlightEntry (gCurrentPane.currIndex);		-- need this for when called from onVerticalScroll
end

-----------------------------------------

function Atr_ShowHistory ()

	Atr_Col1_Heading:Hide();
	Atr_Col3_Heading:Hide();
	Atr_Col4_Heading:Hide();

	Atr_Col3_Heading:SetText ("History");

	local numrows = #gCurrentPane.sortedHist;

--ZC_chatmsg ("gCurrentPane.sortedHist: "..numrows,1,0,0);

	if (numrows > 0) then
		Atr_Col1_Heading:Show();
		Atr_Col3_Heading:Show();
	end

	local line;							-- 1 through 12 of our window to scroll
	local dataOffset;					-- an index into our data calculated from the scroll offset

	FauxScrollFrame_Update (AuctionatorScrollFrame, numrows, 12, 16);

	for line = 1,12 do

		dataOffset = line + FauxScrollFrame_GetOffset (AuctionatorScrollFrame);

		local lineEntry = getglobal ("AuctionatorEntry"..line);

		lineEntry:SetID(dataOffset);

		if (dataOffset <= numrows and gCurrentPane.sortedHist[dataOffset]) then

			local data = gCurrentPane.sortedHist[dataOffset];

			local lineEntry_item_tag = "AuctionatorEntry"..line.."_PerItem_Price";

			local lineEntry_item		= getglobal(lineEntry_item_tag);
			local lineEntry_itemtext	= getglobal("AuctionatorEntry"..line.."_PerItem_Text");
			local lineEntry_text		= getglobal("AuctionatorEntry"..line.."_EntryText");
			local lineEntry_stack		= getglobal("AuctionatorEntry"..line.."_StackPrice");

			lineEntry_item:Show();
			lineEntry_itemtext:Hide();
			lineEntry_stack:SetText	("");

			MoneyFrame_Update (lineEntry_item_tag, ZC_round(data.itemPrice) );

			lineEntry_text:SetText (Atr_BuildHistItemText (data));
			lineEntry_text:SetTextColor (0.8, 0.8, 1.0);

			lineEntry:Show();
		else
			lineEntry:Hide();
		end
	end

	Atr_HighlightEntry (gCurrentPane.histIndex);		-- need this for when called from onVerticalScroll
end

-----------------------------------------

function Atr_FindBestCurrentAuction()

	if (Atr_IsModeCreateAuction()) then
--		gCurrentPane.currIndex = gCurrentPane.activeScan:FindMatchByStackSize (gCurrentPane.stackSize);
		gCurrentPane.currIndex = gCurrentPane.activeScan:FindCheapest ();
	elseif (Atr_IsModeBuy()) then
		gCurrentPane.currIndex = gCurrentPane.activeScan:FindCheapest ();
	else
		gCurrentPane.currIndex = gCurrentPane.activeScan:FindMatchByYours ();
	end

end

-----------------------------------------

function Atr_FindBestHistoricalAuction()

	gCurrentPane.histIndex = nil;

	if (gCurrentPane.sortedHist and #gCurrentPane.sortedHist > 0) then
		gCurrentPane.histIndex = 1;
	end
end

-----------------------------------------

function Atr_HighlightEntry(entryIndex)
	local line;				-- 1 through 12 of our window to scroll

	for line = 1,12 do

		local lineEntry = getglobal ("AuctionatorEntry"..line);

		if (lineEntry:GetID() == entryIndex) then
			lineEntry:SetButtonState ("PUSHED", true);
		else
			lineEntry:SetButtonState ("NORMAL", false);
		end
	end

	local doEnableCancel = false;
	local doEnableBuy = false;
	local data;
	
	if (Atr_ShowingCurrentAuctions() and entryIndex ~= nil and entryIndex > 0 and entryIndex <= #gCurrentPane.activeScan.sortedData) then
		data = gCurrentPane.activeScan.sortedData[entryIndex];
		if (data.yours) then
			doEnableCancel = true;
		end
		
		if (not data.yours and data.buyoutPrice > 0) then
			doEnableBuy = true;
		end
	end

	Atr_Buy1_Button:Show();
	Atr_Buy1_Button:Enable();
	Atr_BatchCancelButton:Hide();
	
	if (doEnableCancel) then
		Atr_Buy1_Button:Hide();
		Atr_BatchCancelButton:Show();

		if (data.count == 1) then
			Atr_BatchCancelButton:SetText ("Cancel Auction");
		else
			Atr_BatchCancelButton:SetText ("Cancel Auctions");
		end
	elseif (not doEnableBuy) then
		Atr_Buy1_Button:Disable();
	end
	
end

-----------------------------------------

function Atr_EntryOnClick()

	if (Atr_Batch_Frame:IsShown()) then
		return;
	end

	local entryIndex = this:GetID();

	if (Atr_ShowingCurrentAuctions()) then
		gCurrentPane.currIndex = entryIndex;
	else
		gCurrentPane.histIndex = entryIndex;
	end

	Atr_HighlightEntry (entryIndex);
	Atr_UpdateRecommendation();


	PlaySound ("igMainMenuOptionCheckBoxOn");
end

-----------------------------------------

function AuctionatorMoneyFrame_OnLoad()

	this.small = 1;
	MoneyFrame_SetType(this, "AUCTION");
end


-----------------------------------------

function Atr_ShowBatchFrame()

	gAutoSell_ItemName			= gCurrentPane.itemName;
	gAutoSell_ItemLink			= gCurrentPane.itemLink;
	gAutoSell_GoodStackSize		= gCurrentPane.stackSize;
	gAutoSell_StartPrice		= MoneyInputFrame_GetCopper(StartPrice);
	gAutoSell_BuyoutPrice		= MoneyInputFrame_GetCopper(BuyoutPrice);
	gAutoSell_Hours				= AuctionFrameAuctions.duration / 60;

	local _, _, _, _, _, _, _, iStackCount = GetItemInfo (gAutoSell_ItemName);

	gAutoSell_FullStackSize = iStackCount;
	gAutoSell_TotalItems	= 0;

	for bagID = kFirstBag, kLastBag do
		numslots = GetContainerNumSlots (bagID);
		for slotID = 1,numslots do
			local itemLink = GetContainerItemLink(bagID, slotID);
			if (itemLink) then
				local itemName				= GetItemInfo(itemLink);
				local texture, itemCount	= GetContainerItemInfo(bagID, slotID);

				if (itemName == gAutoSell_ItemName) then
					gAutoSell_TotalItems = gAutoSell_TotalItems + itemCount;
				end
			end
		end
	end

	local numStacks = math.floor (gAutoSell_TotalItems / gAutoSell_GoodStackSize);

	Atr_SetTextureButton ("Atr_Batch_Texture", gAutoSell_GoodStackSize);

--	MoneyFrame_Update ("Atr_Batch_Start_Price",  gAutoSell_StartPrice);
--	MoneyFrame_Update ("Atr_Batch_Buyout_Price", gAutoSell_BuyoutPrice);

--	Atr_Batch_Duration:SetText (gAutoSell_Hours.." hours");
	Atr_Batch_Itemname:SetText (gAutoSell_ItemName.." x "..gAutoSell_GoodStackSize);

	Atr_Batch_NumAuctions:SetText (numStacks);

	Atr_Batch_Frame:Show();
end



-----------------------------------------

function Atr_BatchCancel()

--	ZC_chatmsg2 ("gCurrentPane.currIndex: ", gCurrentPane.currIndex);
--	ZC_chatmsg2 ("gCurrentPane.histIndex: ", gCurrentPane.histIndex);

	if (not Atr_ShowingCurrentAuctions()) then
		return;
	end
	
	local data = gCurrentPane.activeScan.sortedData[gCurrentPane.currIndex];

	if (not data.yours) then
		return;
	end

	local i = 1;

	while (true) do
		local name, texture, count, quality, canUse, level,
		minBid, minIncrement, buyoutPrice, bidAmount,
		highBidder, owner = GetAuctionItemInfo ("owner", i);

		if (name == nil) then
			break;
		end

		if (name == gCurrentPane.activeScan.itemName and buyoutPrice == data.buyoutPrice and count == data.stackSize) then
			CancelAuction (i);
			AuctionatorSubtractFromScan (name, count, buyoutPrice);
			gJustPosted_ItemName = nil;
		end

		i = i + 1;
	end

end

-----------------------------------------

function Atr_DropDown1_OnLoad (self)
	UIDropDownMenu_Initialize(self, Atr_DropDown1_Initialize);
	UIDropDownMenu_SetSelectedValue(Atr_DropDown1, MODE_LIST_ACTIVE);
	Atr_DropDown1:Show();
end

-----------------------------------------

function Atr_DropDown1_Initialize()
	local info = UIDropDownMenu_CreateInfo();
	
	info.text = "Active Items";
	info.value = MODE_LIST_ACTIVE;
	info.func = Atr_DropDown1_OnClick;
	info.checked = nil;
	UIDropDownMenu_AddButton(info);

	info.text = "All Items";
	info.value = MODE_LIST_ALL;
	info.func = Atr_DropDown1_OnClick;
	info.checked = nil;
	UIDropDownMenu_AddButton(info);

end

-----------------------------------------

function Atr_DropDown1_OnClick(self)
	
	UIDropDownMenu_SetSelectedValue(Atr_DropDown1, self.value);
	
	local mode = self.value;
	
	if (mode == MODE_LIST_ALL) then
		Atr_DisplayHlist();
	end
	
	if (mode == MODE_LIST_ACTIVE) then
		Atr_DisplayHlist();
	end
	
end


-----------------------------------------

function Atr_ShowOptionsFrame()

	AuctionatorOptionsFrame:Show();
	AuctionatorOptionsFrame:SetBackdropColor(0,0,0,100);

	AuctionatorConfigFrameTitle:SetText ("Auctionator Options for "..UnitName("player"));

	local expText = "<html><body>"
					.."<h1>What is Auctionator?</h1><br/>"
					.."<p>"
					.."Figuring out a good buyout price when posting auctions can be tedious and time-consuming.  If you're like most people, you first browse the current "
					.."auctions to get a sense of how much your item is currently selling for.  Then you undercut the lowest price by a bit.  If you're creating multiple auctions "
					.."you're bouncing back and forth between the Browse tab and the Auctions tab, doing lots of division in "
					.."your head, and doing lots of clicking and typing."
					.."</p><br/><h1>How it works</h1><br/><p>"
					.."Auctionator makes this whole process easy and streamlined.  When you select an item to auction, Auctionator displays a summary of all the current auctions for "
					.."that item sorted by per-item price.  Auctionator also calculates a recommended buyout price based on the cheapest per-item price for your item.  If you're "
					.."selling a stack rather than a single item, Auctionator bases its recommended buyout price on the cheapest stack of the same size."
					.."</p><br/><p>"
					.."If you don't like Auctionator's recommendation, you can click on any line in the summary and Auctionator will recalculate the recommended buyout price based "
					.."on that auction.  Of course, you can always override Auctionator's recommendation by just typing in your own buyout price."
					.."</p><br/><p>"
					.."With Auctionator, creating an auction is usually just a matter of picking an item to auction and clicking the Create Auction button."
					.."</p>"
					.."</body></html>"
					;



	AuctionatorDescriptionHTML:SetText (expText);
	AuctionatorDescriptionHTML:SetSpacing (3);

	AuctionatorVersionText:SetText ("Version: "..AuctionatorVersion);

	AuctionatorOption_Enable_Alt_CB:SetChecked		(ZC_NumToBool(AUCTIONATOR_ENABLE_ALT));
	AuctionatorOption_Open_SELL_CB:SetChecked		(ZC_NumToBool(AUCTIONATOR_OPEN_FIRST));
	AuctionatorOption_Open_BUY_CB:SetChecked		(ZC_NumToBool(AUCTIONATOR_OPEN_BUY));
	AuctionatorOption_Open_All_Bags_CB:SetChecked	(ZC_NumToBool(AUCTIONATOR_OPEN_ALL_BAGS));

	AuctionatorOption_Def_Duration_CB:SetChecked (AUCTIONATOR_DEF_DURATION == "S" or AUCTIONATOR_DEF_DURATION == "M" or AUCTIONATOR_DEF_DURATION == "L");

	Atr_SetDurationOptionRB (AUCTIONATOR_DEF_DURATION);
end


-----------------------------------------

function Atr_SetDurationOptionRB(name)

	Atr_RB_S:SetChecked (ZC_StringEndsWith (name, "S"));
	Atr_RB_M:SetChecked (ZC_StringEndsWith (name, "M"));
	Atr_RB_L:SetChecked (ZC_StringEndsWith (name, "L"));

end

-----------------------------------------

function AuctionatorBasicOptionsSave()

	AUCTIONATOR_ENABLE_ALT		= ZC_BoolToNum(AuctionatorOption_Enable_Alt_CB:GetChecked ());
	AUCTIONATOR_OPEN_FIRST		= ZC_BoolToNum(AuctionatorOption_Open_SELL_CB:GetChecked ());
	AUCTIONATOR_OPEN_BUY		= ZC_BoolToNum(AuctionatorOption_Open_BUY_CB:GetChecked ());
	AUCTIONATOR_OPEN_ALL_BAGS	= ZC_BoolToNum(AuctionatorOption_Open_All_Bags_CB:GetChecked ());

	AUCTIONATOR_DEF_DURATION = "N";

	if (AuctionatorOption_Def_Duration_CB:GetChecked()) then
		if (Atr_RB_S:GetChecked())	then	AUCTIONATOR_DEF_DURATION = "S"; end;
		if (Atr_RB_M:GetChecked())	then	AUCTIONATOR_DEF_DURATION = "M"; end;
		if (Atr_RB_L:GetChecked())	then	AUCTIONATOR_DEF_DURATION = "L"; end;
	end
end

-----------------------------------------

function Atr_ShowBasicOptionsFrame()

	AuctionatorBasicOptionsFrame:Show();

end

-----------------------------------------

function Atr_Option_OnClick (self)

	if (ZC_StringContains (self:GetName(), "Open_BUY") and self:GetChecked()) then
		AuctionatorOption_Open_SELL_CB:SetChecked (false);
	end

	if (ZC_StringContains (self:GetName(), "Open_SELL") and self:GetChecked()) then
		AuctionatorOption_Open_BUY_CB:SetChecked (false);
	end

end

-----------------------------------------

function Atr_UCConfigLine_Setup(thresh, lineText)

	getglobal("UC_"..thresh.."_RangeText"):SetText (lineText);

	MoneyInputFrame_SetCopper (getglobal("UC_"..thresh.."_MoneyInput"), AUCTIONATOR_SAVEDVARS["_"..thresh]);

end

-----------------------------------------

function Atr_ShowUCConfigFrame()

	Atr_UCConfigLine_Setup (5000000,	"over 500 gold");
	Atr_UCConfigLine_Setup (1000000,	"over 100 gold");
	Atr_UCConfigLine_Setup (200000,		"over 20 gold");
	Atr_UCConfigLine_Setup (50000,		"over 5 gold");
	Atr_UCConfigLine_Setup (10000,		"over 1 gold");
	Atr_UCConfigLine_Setup (2000,		"over 20 silver");
	Atr_UCConfigLine_Setup (500,		"over 5 silver");

	Atr_Starting_Discount:SetText (AUCTIONATOR_SAVEDVARS.STARTING_DISCOUNT);


--	AuctionatorOptionsFrame:Hide();
	AuctionatorUndercuttingConfig:Show();
--	Atr_Mask:Show();


end


-----------------------------------------

function Atr_UCConfigLine_Save(thresh)

	AUCTIONATOR_SAVEDVARS["_"..thresh]	= MoneyInputFrame_GetCopper(getglobal("UC_"..thresh.."_MoneyInput"));

end

-----------------------------------------

function AuctionatorUCConfigSave()

	Atr_UCConfigLine_Save (5000000);
	Atr_UCConfigLine_Save (1000000);
	Atr_UCConfigLine_Save (200000);
	Atr_UCConfigLine_Save (50000);
	Atr_UCConfigLine_Save (10000);
	Atr_UCConfigLine_Save (2000);
	Atr_UCConfigLine_Save (500);

	AUCTIONATOR_SAVEDVARS.STARTING_DISCOUNT = Atr_Starting_Discount:GetNumber ();

end


-----------------------------------------

function Atr_ShowOptionTooltip (elem)

	local name = elem:GetName();
	local titleText;
	local text;

	if (ZC_StringContains (name, "Enable_Alt")) then
		titleText = "Enable alt-key shortcut";
		text = "If this option is checked, holding the Alt key down while clicking an item in your bags will switch to the Auctionator panel, place the item in the Auction Item area, and start the scan.";
	end

	if (ZC_StringContains (name, "Open_SELL")) then
		titleText = "Automatically open Auctionator SELL panel";
		text = "If this option is checked, the Auctionator SELL panel will display first whenever you open the Auction House window.";
	end

	if (ZC_StringContains (name, "Open_BUY")) then
		titleText = "Automatically open Auctionator BUY panel";
		text = "If this option is checked, the Auctionator BUY panel will display first whenever you open the Auction House window.";
	end

	if (ZC_StringContains (name, "Open_All_Bags")) then
		titleText = "Automatically open all bags";
		text = "If this option is checked, ALL your bags will be opened when you first open the Auctionator panel.";
	end

	if (ZC_StringContains (name, "Def_Duration")) then
		titleText = "Set a default duration";
		text = "If this option is checked, every time you initiate a new auction the auction duration will be reset to the default duration you've selected.";
	end




	GameTooltip:SetOwner(this, "ANCHOR_BOTTOM");
	GameTooltip:SetText(titleText, 0.9, 1.0, 1.0);
	GameTooltip:AddLine(text, 0.5, 0.5, 1.0, 1);
	GameTooltip:Show();

end



-----------------------------------------

function Atr_IsTabSelected(whichTab)

	if (not AuctionFrame or not AuctionFrame:IsShown()) then
		return false;
	end

	if (not whichTab) then
		return (Atr_IsTabSelected(SELL_TAB) or Atr_IsTabSelected(MORE_TAB) or Atr_IsTabSelected(BUY_TAB));
	end

	return (PanelTemplates_GetSelectedTab (AuctionFrame) == Atr_FindTabIndex(whichTab));
end

-----------------------------------------

function Atr_IsAuctionatorTab (tabIndex)

	if (tabIndex == Atr_FindTabIndex(SELL_TAB) or tabIndex == Atr_FindTabIndex(MORE_TAB) or tabIndex == Atr_FindTabIndex(BUY_TAB) ) then

		return true;

	end

	return false;
end

-----------------------------------------

function Atr_Confirm_Yes()

	if (Atr_Confirm_Proc_Yes) then
		Atr_Confirm_Proc_Yes();
		Atr_Confirm_Proc_Yes = nil;
	end

	Atr_Confirm_Frame:Hide();

end


-----------------------------------------

function Atr_Confirm_No()

	Atr_Confirm_Frame:Hide();

end


-----------------------------------------

function Atr_AddHistoricalPrice (itemName, price, stacksize, itemLink, testwhen)

	if (not AUCTIONATOR_PRICING_HISTORY[itemName] ) then
		AUCTIONATOR_PRICING_HISTORY[itemName] = {};
	end

	local itemId, suffixId, uniqueId = ZC_ItemIDfromLink (itemLink);

	local is = itemId;

	if (suffixId ~= 0) then
		is = is..":"..suffixId;
		if (tonumber(suffixId) < 0) then
			is = is..":"..uniqueId;
		end
	end

	AUCTIONATOR_PRICING_HISTORY[itemName]["is"]  = is;

	local hist = tostring (ZC_round (price))..":"..stacksize;

	local roundtime = floor (time() / 60) * 60;		-- so multiple auctions close together don't generate too many entries

	local tag = tostring(ToTightTime(roundtime));

	if (testwhen) then
		tag = tostring(ToTightTime(testwhen));
	end

	AUCTIONATOR_PRICING_HISTORY[itemName][tag] = hist;

end

-----------------------------------------

function Atr_HasHistoricalData (itemName)

	if (AUCTIONATOR_PRICING_HISTORY[itemName] ) then
		return true;
	end

	return false;
end


-----------------------------------------

function Atr_BuildGlobalHistoryList(full)

	gHistoryItemList	= {};
	
	local n = 1;

	if (full) then
		for name,hist in pairs (AUCTIONATOR_PRICING_HISTORY) do
			gHistoryItemList[n] = name;
			n = n + 1;
		end
	else
		local name = "";
		while (name) do
			name = Atr_GetActiveAuction(n);
			if (name) then
				gHistoryItemList[n] = name;
				n = n + 1;
			end
		end
	end
	
	table.sort (gHistoryItemList);

end



-----------------------------------------

function Atr_Buy1_Onclick ()

	if (not Atr_ShowingCurrentAuctions()) then
		return;
	end
	
	local scan = gCurrentPane.activeScan;
	
	local data = scan.sortedData[gCurrentPane.currIndex];

	local numBids = Atr_Buy_Stack (gCurrentPane.activeScan.itemName, data.buyoutPrice, data.stackSize, false);

	if (numBids == 0) then
		gActionState = ATR_BUYING;

		local queryString = scan.itemName;

		if (scan.itemSoftName) then
			queryString = scan.itemSoftName;
		end

		queryString = string.sub (queryString,0,31);	-- attempting to reduce number of disconnects

		QueryAuctionItems (queryString, "", "", nil, scan.itemClass, scan.itemSubclass, data.pagenum, nil, nil);
	end

end

-----------------------------------------

function Atr_Continue_Buying ()

	gActionState = ATR_NOTHING;

	local data = gCurrentPane.activeScan.sortedData[gCurrentPane.currIndex];

	local numBids = Atr_Buy_Stack (gCurrentPane.itemName, data.buyoutPrice, data.stackSize, false);

end

-----------------------------------------

local gAtr_Buy_Index;
local gAtr_Buy_BuyoutPrice;
local gAtr_Buy_ItemName;
local gAtr_Buy_StackSize;

StaticPopupDialogs["AUCTIONATOR_BUYOUT_AUCTION"] = {
	text = BUYOUT_AUCTION_CONFIRMATION,
	button1 = ACCEPT,
	button2 = CANCEL,
	OnAccept = function(self)
		local v = PlaceAuctionBid("list", gAtr_Buy_Index, gAtr_Buy_BuyoutPrice);
		gActionState = ATR_JUST_BOUGHT;
	end,
	OnShow = function(self)
		MoneyFrame_Update(self.moneyFrame, gAtr_Buy_BuyoutPrice);
	end,
	hasMoneyFrame = 1,
	showAlert = 1,
	timeout = 0,
	exclusive = 1,
	hideOnEscape = 1
};

-----------------------------------------

function Atr_Buy_Stack (targetItemName, targetBuyoutPrice, targetStackSize, buyAll)

	local numBids = 0;
	local i		  = 1;

	while (true) do
		local name, texture, count, quality, canUse, level,
		minBid, minIncrement, buyoutPrice, bidAmount,
		highBidder, owner = GetAuctionItemInfo ("list", i);

		if (name == nil) then
			break;
		end

		if (ZC_StringSame (name, targetItemName) and buyoutPrice == targetBuyoutPrice and count == targetStackSize) then
			
			gAtr_Buy_BuyoutPrice	= buyoutPrice;
			gAtr_Buy_Index			= i;
			gAtr_Buy_ItemName		= name;
			gAtr_Buy_StackSize		= count;
			
			StaticPopup_Show("AUCTIONATOR_BUYOUT_AUCTION");		
			
			if (not buyAll) then
				return 1;
			end
			
			numBids = numBids + 1;
		end

		i = i + 1;
	end

	return numBids;
	
end

-----------------------------------------

function Atr_AdjustScanForBoughtItem()

	gActionState = ATR_NOTHING;
	AuctionatorSubtractFromScan (gAtr_Buy_ItemName, gAtr_Buy_StackSize, gAtr_Buy_BuyoutPrice)

end

-----------------------------------------

function Atr_Item_Autocomplete(self)

	local text = self:GetText();
	local textlen = strlen(text);
	local name;

	-- first search shopping lists

	local numLists = #AUCTIONATOR_SHOPPING_LISTS;
	local n;
	
	for n = 1,numLists do
		local slist = AUCTIONATOR_SHOPPING_LISTS[n];

		local numItems = #slist.items;

		if ( numItems > 0 ) then
			for i=1, numItems do
				name = slist.items[i];
				if ( name and text and (strfind(strupper(name), strupper(text), 1, 1) == 1) ) then
					self:SetText(name);
					if ( self:IsInIMECompositionMode() ) then
						self:HighlightText(textlen - strlen(arg1), -1);
					else
						self:HighlightText(textlen, -1);
					end
					return;
				end
			end
		end
	end
	

	-- next search history list

	numItems = #gHistoryItemList;

	if ( numItems > 0 ) then
		for i=1, numItems do
			name = gHistoryItemList[i];
			if ( name and text and (strfind(strupper(name), strupper(text), 1, 1) == 1) ) then
				self:SetText(name);
				if ( self:IsInIMECompositionMode() ) then
					self:HighlightText(textlen - strlen(arg1), -1);
				else
					self:HighlightText(textlen, -1);
				end
				return;
			end
		end
	end
end

-----------------------------------------

function Atr_GetCurrentPane ()			-- so other modules can use gCurrentPane
	return gCurrentPane;
end

-----------------------------------------

function Atr_SetUINeedsUpdate ()			-- so other modules can easily set
	gCurrentPane.UINeedsUpdate = true;
end


-----------------------------------------

function Atr_CalcNewPrice (price)

	if	(price > 5000000)	then return roundPriceDown (price, AUCTIONATOR_SAVEDVARS._5000000);	end;
	if	(price > 1000000)	then return roundPriceDown (price, AUCTIONATOR_SAVEDVARS._1000000);	end;
	if	(price >  200000)	then return roundPriceDown (price, AUCTIONATOR_SAVEDVARS._200000);	end;
	if	(price >   50000)	then return roundPriceDown (price, AUCTIONATOR_SAVEDVARS._50000);	end;
	if	(price >   10000)	then return roundPriceDown (price, AUCTIONATOR_SAVEDVARS._10000);	end;
	if	(price >    2000)	then return roundPriceDown (price, AUCTIONATOR_SAVEDVARS._2000);	end;
	if	(price >     500)	then return roundPriceDown (price, AUCTIONATOR_SAVEDVARS._500);		end;
	if	(price >       0)	then return math.floor (price - 1);	end;

	return 0;
end

-----------------------------------------

function Atr_AbbrevItemName (itemName)

	return string.gsub (itemName, "Scroll of Enchant", "SoE");

end


-----------------------------------------
-- roundPriceDown - rounds a price down to the next lowest multiple of a.
--				  - if the result is not at least a/2 lower, rounds down by a again.
--
--	examples:  	(128790, 500)  ->  128500
--				(128700, 500)  ->  128000
--				(128400, 500)  ->  128000
-----------------------------------------

function roundPriceDown (price, a)

	local newprice = math.floor(price / a) * a;

	if ((price - newprice) < a/2) then
		newprice = newprice - a;
	end

	if (newprice == price) then
		newprice = newprice - 1;
	end

	return newprice;

end

-----------------------------------------

function ToTightTime(t)

	return floor((t - gTimeTightZero)/60);

end

-----------------------------------------

function FromTightTime(tt)

	return (tt*60) + gTimeTightZero;

end





